package com.cuit.solarenergy.minio;


import com.cuit.solarenergy.utils.AssertUtil;
import com.cuit.solarenergy.utils.ErrorMessage;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.activation.MimetypesFileTypeMap;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.*;

/**
 * @author wzw
 * @Describe minio对外服务
 */
@Component
@Slf4j
public class MinIoServerImpl implements MinioServer{
    @Autowired
    private MinIoUtil minIoUtil;
    @Resource
    private MinioConfig minioConfig;

    /**
     * 以File类形式上传文件
     *
     * @param file         文件对象
     * @param fileName     文件存储名称
     * @param onlyFlag     文件名是否需要唯一 默认为 true - 需要唯一
     * @return minio对象唯一id
     */
    @Override
    public String uploadFile(File file, String fileName, Boolean onlyFlag) {
        String filePathName = minIoUtil.getMinoDatePathFileName("", fileName, onlyFlag);
        return uploadFile(file, filePathName, fileName, onlyFlag);
    }


    /**
     * 以File类形式上传文件
     *
     * @param file         文件对象
     * @param filePath     文件存储路径，为空未指定时按日期自动生成
     * @param fileName     文件存储名称
     * @param onlyFlag     文件名是否需要唯一 默认为 true - 需要唯一
     * @return minio对象唯一id
     */
    @Override
    public String uploadFile(File file, String filePath, String fileName, Boolean onlyFlag) {

        String bucketName = minioConfig.getBucketName();
        //1.格式校验
        AssertUtil.isTrue(file == null, "文件流为空！");
        AssertUtil.isTrue(filePath == null || "".equals(filePath), "文件路径为空！");
        if (fileName == null || "".equals(fileName)) {
            fileName = file.getName();
        }
        String filePathName = "";
        if (StringUtils.isEmpty(filePath)) {
            //2.路径判断,未传路径则默认按当前时分秒作为路径     文件名唯一性处理
            filePathName = minIoUtil.getMinoDatePathFileName("", fileName, onlyFlag);
        } else {
            //文件名唯一性处理
            fileName = minIoUtil.onlyParse(fileName, onlyFlag);
            String filePathNew = filePath.replace("\\", "/");
            if (!filePathNew.endsWith("/")) {
                filePathNew += "/";
            }
            filePathName = filePathNew + fileName;
        }
        //4.上传文件
        try {
            String objectName = minIoUtil.upload(bucketName, filePathName, file);
            return objectName;
        } catch (Exception e) {
            throw new ErrorMessage("文件上传失败！");
        }
    }


    /**
     * 以MultipartFile类形式上传文件
     *
     * @param file         文件对象
     * @param fileName     文件存储名称
     * @param onlyFlag     文件名是否需要唯一 默认为 true - 需要唯一
     * @return minio对象唯一id
     */
    @Override
    public String uploadFile(MultipartFile file, String fileName, Boolean onlyFlag) {
        String filePathName = minIoUtil.getMinoDatePathFileName("", fileName, onlyFlag);
        return uploadFile(file, filePathName, fileName, onlyFlag);
    }


    /**
     * 以MultipartFile类形式上传文件
     *
     * @param file         文件对象
     * @param filePath     文件存储路径，为空未指定时按日期自动生成
     * @param fileName     文件存储名称
     * @param onlyFlag     文件名是否需要唯一 默认为 true - 需要唯一
     * @return minio对象唯一id
     */
    @Override
    public String uploadFile(MultipartFile file, String filePath, String fileName, Boolean onlyFlag) {

        String bucketName = minioConfig.getBucketName();
        //1.格式校验
        AssertUtil.isTrue(file == null, "文件流为空！");
        AssertUtil.isTrue(filePath == null || "".equals(filePath), "文件路径为空！");
        if (fileName == null || "".equals(fileName)) {
            fileName = file.getName();
        }
        String filePathName = "";
        if (StringUtils.isEmpty(filePath)) {
            //2.路径判断,未传路径则默认按当前时分秒作为路径     文件名唯一性处理
            filePathName = minIoUtil.getMinoDatePathFileName("", fileName, onlyFlag);
        } else {
            //文件名唯一性处理
            fileName = minIoUtil.onlyParse(fileName, onlyFlag);
            String filePathNew = filePath.replace("\\", "/");
            if (!filePathNew.endsWith("/")) {
                filePathNew += "/";
            }
            filePathName = filePathNew + fileName;
        }
        //4.上传文件
        try {
            String objectName = minIoUtil.upload(bucketName, filePathName, file);
            return objectName;
        } catch (Exception e) {
            throw new ErrorMessage("文件上传失败！");
        }
    }


//    /**
//     * 获取原图可查看图片链接
//     *
//     * @param objectName
//     * @param expireDayCount
//     * @return
//     */
//    @Override
//    public String getShareURL(String objectName, int expireDayCount) {
//        String bucketName = minioConfig.getBucketName();
//        //校验当前文件是否存在
//        AssertUtil.isTrue(!minIoUtil.fileExists(objectName, bucketName),"当前路径所指文件在minio中不存在！");
//        String url = "";
//        try {
//            url = minIoUtil.getPresignedObjectUrl(objectName, bucketName, expireDayCount);
//        }catch (Exception e) {
//            throw new ErrorMessage("url获取失败！");
//        }
//        return url;
//    }

//    /**
//     * 获取压缩图可查看图片链接
//     *
//     * @param objectName
//     * @param expireDayCount
//     * @return
//     */
//    @Override
//    public String getCompressShareURL(String objectName, int expireDayCount) {
//        String bucketName = minioConfig.getBucketName();
//        //校验当前文件是否存在
//        AssertUtil.isTrue(!minIoUtil.fileExists("ys_" + objectName, bucketName),"当前路径所指文件在minio中不存在！");
//        String url = "";
//        try {
//            url = minIoUtil.getPresignedObjectUrl("ys_" + objectName, bucketName,expireDayCount);
//        }catch (Exception e) {
//            throw new ErrorMessage("url获取失败！");
//        }
//        return url;
//    }


    /**
     * 获取原图可查看图片链接
     *
     * @param objectName
     * @return
     */
//    @Override
//    public String getShareURL(String objectName) {
//        String bucketName = minioConfig.getBucketName();
//        //校验当前文件是否存在
//        AssertUtil.isTrue(!minIoUtil.fileExists(objectName, bucketName),"当前路径所指文件在minio中不存在！");
//        String url = "";
//        try {
//            url = minIoUtil.getPresignedObjectUrl(objectName, bucketName, null);
//        }catch (Exception e) {
//            throw new ErrorMessage("url获取失败！");
//        }
//        return url;
//    }

    @Override
    public String getCompressPermanentShareURL(String objectName) {
        String bucketName = minioConfig.getBucketName();
        //校验当前文件是否存在
        AssertUtil.isTrue(!minIoUtil.fileExists("ys_" + objectName, bucketName),"当前路径所指文件在minio中不存在！");
        String url = "";
        try {
            url = minioConfig.getVisitPoint()
                    + "/"
                    + bucketName
                    + "/ys_"
                    + objectName;
        }catch (Exception e) {
            throw new ErrorMessage("url获取失败！");
        }
        return url;
    }

    @Override
    public String getPermanentShareURL(String objectName) {
        String bucketName = minioConfig.getBucketName();
        //校验当前文件是否存在
        AssertUtil.isTrue(!minIoUtil.fileExists(objectName, bucketName),"当前路径所指文件在minio中不存在！");
        String url = "";
        try {
            url = minioConfig.getVisitPoint()
                    + "/"
                    + bucketName
                    + "/"
                    + objectName;
        }catch (Exception e) {
            throw new ErrorMessage("url获取失败！");
        }
        return url;
    }


    @Override
    public String getPermanentShareURLWithoutVisitPoint(String objectName) {
        String bucketName = minioConfig.getBucketName();
        //校验当前文件是否存在
        AssertUtil.isTrue(!minIoUtil.fileExists(objectName, bucketName), "当前路径所指文件在minio中不存在！");
        String url = "";
        try {
            url = bucketName
                    + "/"
                    + objectName;
        } catch (Exception e) {
            throw new ErrorMessage("url获取失败！");
        }
        return url;
    }

//    /**
//     * 获取压缩图可查看图片链接
//     *
//     * @param objectName
//     * @return
//     */
//    @Override
//    public String getCompressShareURL(String objectName) {
//        String bucketName = minioConfig.getBucketName();
//        //校验当前文件是否存在
//        AssertUtil.isTrue(!minIoUtil.fileExists("ys_" + objectName, bucketName),"当前路径所指文件在minio中不存在！");
//        String url = "";
//        try {
//            url = minIoUtil.getPresignedObjectUrl(objectName, bucketName,null);
//        }catch (Exception e) {
//            throw new ErrorMessage("url获取失败！");
//        }
//        return url;
//    }


//    private String getUrl(String newFileName, String bucketName) {
//        return minioConfig.getVisitPoint()
//                + "/"
//                + bucketName
//                + "/"
//                + newFileName;
//    }


    /**
     * MultipartFile 转 File
     *
     * @param file
     * @throws Exception
     */
    public File multipartFileToFile(MultipartFile file) throws IOException {
        File toFile = null;

        InputStream ins = null;
        try {
            ins = file.getInputStream();
            toFile = new File(file.getOriginalFilename());
            inputStreamToFile(ins, toFile);
            ins.close();
        } finally {
            if (ins != null) {
                try {
                    ins.close();
                }catch (Exception e) {
                    log.error("关闭文件时出错：" + e.getMessage());
                }
            }

        }
        return toFile;
    }


    //获取流文件
    private void inputStreamToFile(InputStream ins, File file) {
        OutputStream os = null;
        try {
            os = new FileOutputStream(file);
            int bytesRead = 0;
            byte[] buffer = new byte[8192];
            while ((bytesRead = ins.read(buffer, 0, 8192)) != -1) {
                os.write(buffer, 0, bytesRead);
            }
            os.flush();

        } catch (Exception e) {
            log.error("获取文件流时出错：" + e.getMessage());
        } finally {
            if (ins != null) {
                try {
                    ins.close();
                } catch (Exception e) {
                    log.error("关闭文件时出错：" + e.getMessage());
                }
            }
            if (os != null) {
                try {
                    os.close();
                } catch (Exception e) {
                    log.error("关闭文件时出错：" + e.getMessage());
                }
            }
        }
    }



    /**
     * 删除文件
     *
     * @param objectName   文件路径（file_path字段）

     * @return
     */
    @Override
    public Boolean deleteFile(String objectName) {
        String bucketName = minioConfig.getBucketName();
        //校验当前文件是否存在
        AssertUtil.isTrue(!minIoUtil.fileExists(objectName, bucketName),"当前路径所指文件" + objectName +"在minio中不存在！");
        try {
            boolean remove = minIoUtil.remove(objectName, bucketName);
            return remove;
        }catch (Exception e) {
            throw new BcException(FileOpeRetCode.DELETE_FILE_EXCEPTION.getCode(),"文件删除失败！");
        }
    }


    /**
     * 下载文件
     * @param objectName 图片名称（file_path）
     * @param savePath 保存路径
     * @return
     */
    @Override
    public Boolean downloadFile(String objectName, String savePath) {
        String bucketName = minioConfig.getBucketName();
        //校验当前文件是否存在
        AssertUtil.isTrue(!minIoUtil.fileExists(objectName, bucketName)," ！");
        try {
            String[] split = objectName.split("/");
            //File saveFile = new File(savePath + split[split.length - 1]);
            if(split[split.length - 1].contains("_")) {
                savePath = savePath + split[split.length - 1].split("_")[1];
            }else {
                savePath = savePath + split[split.length - 1];
            }
            minIoUtil.download(objectName, bucketName,savePath);
        }catch (Exception e) {
            throw new BcException(FileOpeRetCode.DELETE_FILE_EXCEPTION.getCode(),"文件下载失败！");
        }
        return true;
    }


    /**
     * 下载文件，返回可下载链接
     * @param objectName
     * @param response
     */
    @Override
    public void downloadFile(String objectName, HttpServletResponse response) throws  BcException{
        try {
            minIoUtil.download(objectName, minioConfig.getBucketName(), response);
        } catch (Exception e) {
            throw new BcException(FileOpeRetCode.DOWNLOAD_FILE_EXCEPTION.getCode(), "文件下载失败");
        }
    }


    /**
     * 方式二
     * 该方式支持本地文件，也支持http/https远程文件
     *
     * @param fileUrl
     */
    public static String getContentType(String fileUrl) {
        String contentType = null;
        try {
            contentType = new MimetypesFileTypeMap().getContentType(new File(fileUrl));
        } catch (Exception e) {
            e.printStackTrace();
        }
        System.out.println("getContentType, File ContentType is : " + contentType);
        return contentType;
    }

}
