package com.cuit.solarenergy.minio;


import com.cuit.solarenergy.utils.TimeUtil;
import com.cuit.solarenergy.utils.UUIDUtils;
import io.minio.*;
import io.minio.errors.*;
import io.minio.http.Method;
import io.minio.messages.Bucket;
import io.minio.messages.DeleteError;
import io.minio.messages.DeleteObject;
import io.minio.messages.Item;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.util.FastByteArrayOutputStream;
import org.springframework.web.multipart.MultipartFile;

import javax.activation.MimetypesFileTypeMap;
import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.rmi.ServerException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author hpj
 */
@Slf4j
@Component
public class MinIoUtil {
    @Resource
    private MinioClient minioClient;

    /**
     * 查看存储bucket是否存在
     *
     * @return boolean
     */
    public Boolean bucketExists(String bucketName) {
        Boolean found;
        try {
            found = minioClient.bucketExists(BucketExistsArgs.builder().bucket(bucketName).build());
        } catch (Exception e) {
            log.error("Minio操作时出错：判断存储bucket是否存在时：" + e.getMessage() );
            return false;
        }
        return found;
    }

    /**
     * 创建存储bucket
     *
     * @return Boolean
     */
    public Boolean makeBucket(String bucketName) {
        try {
            minioClient.makeBucket(MakeBucketArgs.builder()
                    .bucket(bucketName)
                    .build());
        } catch (Exception e) {
            log.error("Minio操作时出错：创建存储bucket时：" + e.getMessage() );
            return false;
        }
        return true;
    }

    /**
     * 删除存储bucket
     *
     * @return Boolean
     */
    public Boolean removeBucket(String bucketName) {
        try {
            minioClient.removeBucket(RemoveBucketArgs.builder()
                    .bucket(bucketName)
                    .build());
        } catch (Exception e) {
            log.error("Minio操作时出错：删除存储bucket时：" + e.getMessage() );
            return false;
        }
        return true;
    }

    /**
     * 获取全部bucket
     */
    public List<Bucket> getAllBuckets() {
        try {
            return minioClient.listBuckets();
        } catch (Exception e) {
            log.error("Minio操作时出错：获取全部bucket时：" + e.getMessage() );
        }
        return null;
    }


    /**
     * 文件上传
     * @param bucketName 桶名
     * @param fileName 文件名（包括路径）
     * @param file 文件
     * @param contentType 类型
     * @return 对象名，为null时表示失败
     */
    public String upload(String bucketName, String fileName, MultipartFile file, String contentType) {
        try {
            if (!(bucketExists(bucketName))) {
                makeBucket(bucketName);
            }
            PutObjectArgs objectArgs;
            if (StringUtils.isEmpty(contentType)) {
                objectArgs = PutObjectArgs.builder().bucket(bucketName).object(fileName)
                        .stream(file.getInputStream(), file.getSize(), -1).contentType(file.getContentType()).build();
            } else {
                objectArgs = PutObjectArgs.builder().bucket(bucketName).object(fileName)
                        .stream(file.getInputStream(), file.getSize(), -1).contentType(contentType).build();
            }
            ObjectWriteResponse objectWriteResponse = minioClient.putObject(objectArgs);
            if (objectWriteResponse != null) {
                String object = objectWriteResponse.object();
                return object;
            } else {
                return null;
            }
        } catch (Exception e) {
            log.error("Minio操作时出错：文件上传时：" + e.getMessage() );
            e.printStackTrace();
            return null;
        }
    }

    /**
     *
     * 文件上传
     *
     * @param bucketName 桶名
     * @param fileName 文件名（包括路径）
     * @param file 文件
     * @param contentType 类型
     * @return 对象名，为null时表示失败
     */
    public String upload(String bucketName, String fileName, File file, String contentType) {
        try {
            if (!(bucketExists(bucketName))) {
                makeBucket(bucketName);
            }

            FileInputStream fileInputStream = new FileInputStream(file);
            PutObjectArgs objectArgs = null;
            if (StringUtils.isEmpty(contentType)) {
                objectArgs = PutObjectArgs.builder().bucket(bucketName).object(fileName)
                        .stream(fileInputStream, file.length(), -1).contentType(getContentType(file.getAbsolutePath())).build();
            } else {
                objectArgs = PutObjectArgs.builder().bucket(bucketName).object(fileName)
                        .stream(fileInputStream, file.length(), -1).contentType(contentType).build();
            }
            //文件名称相同会覆盖
            ObjectWriteResponse objectWriteResponse = minioClient.putObject(objectArgs);
            if (objectWriteResponse != null) {
                String object = objectWriteResponse.object();
                return object;
            } else {
                return null;
            }
        } catch (Exception e) {
            log.error("Minio操作时出错：文件上传时：" + e.getMessage() );
            e.printStackTrace();
            return null;
        }
    }


    /**
     * 文件上传
     * @param bucketName 桶名
     * @param file 文件
     * @return 对象名，为null时表示失败
     */
    public String upload(String bucketName, MultipartFile file) {
        String originalFilename = file.getOriginalFilename();
        String objectName = getMinoDatePathFileName(null, originalFilename, true);
        try {
            return upload(bucketName, objectName, file, null);
        } catch (Exception e) {
            log.error("Minio操作时出错：文件上传时：" + e.getMessage());
            e.printStackTrace();
            return null;
        }
    }


    /**
     *  文件上传
     * @param bucketName 桶名
     * @param fileName 文件名（包括路径）
     * @param file 文件
     * @return 对象名，为null时表示失败
     */
    public String upload(String bucketName, String fileName, File file) {
        try {
            return upload(bucketName, fileName, file, null);
        } catch (Exception e) {
            log.error("Minio操作时出错：文件上传时：" + e.getMessage() );
            e.printStackTrace();
            return null;
        }
    }

    /**
     *  文件上传
     * @param bucketName 桶名
     * @param fileName 文件名（包括路径）
     * @param file 文件
     * @return 对象名，为null时表示失败
     */
    public String upload(String bucketName, String fileName, MultipartFile file) {
        try {
            return upload(bucketName, fileName, file, null);
        } catch (Exception e) {
            log.error("Minio操作时出错：文件上传时：" + e.getMessage() );
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 文件上传
     * @param bucketName 桶名
     * @param fileName 文件名（包括路径）
     * @param contentType 类型
     * @return 对象名，为null时表示失败
     */
    public String upload(String bucketName, String fileName, String contentType) {
        try {
            File file = new File(fileName);
            String objectName = getMinoDatePathFileName(null, fileName, true);
            return upload(bucketName, objectName, file, contentType);
        } catch (Exception e) {
            log.error("Minio操作时出错：文件上传时：" + e.getMessage() );
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 文件上传
     * @param bucketName 桶名
     * @param fileName 文件名（包括路径）
     * @return
     */
    public String upload(String bucketName, String fileName) {
        try {
            File file = new File(fileName);
            String objectName = getMinoDatePathFileName(null, fileName, true);
            return upload(bucketName, objectName, file,null);
        } catch (Exception e) {
            log.error("Minio操作时出错：文件上传时：" + e.getMessage() );
            e.printStackTrace();
            return null;
        }
    }


    /**
     * 方式二
     * 该方式支持本地文件，也支持http/https远程文件
     *
     * @param fileUrl
     */
    public static String getContentType(String fileUrl) {
        String contentType = null;
        try {
            contentType = new MimetypesFileTypeMap().getContentType(new File(fileUrl));
        } catch (Exception e) {
            log.error("Minio操作时出错：获取文件时：" + e.getMessage() );
            e.printStackTrace();
        }
        log.debug("getContentType, File ContentType is : " + contentType);
        return contentType;
    }


    /**
     * 预览图片
     *
     * @param fileName
     * @return
     */
    public String preview(String fileName, String bucketName) {
        // 查看文件地址
        GetPresignedObjectUrlArgs build = GetPresignedObjectUrlArgs
                .builder().bucket(bucketName)
                .object(fileName)
                .method(Method.GET).build();
        try {
            String url = minioClient.getPresignedObjectUrl(build);
            return url;
        } catch (Exception e) {
            log.error("Minio操作时出错：预览图片：" + e.getMessage() );
        }
        return null;
    }

    /**
     * 文件下载
     *
     * @param fileName 文件名称
     * @param res      response
     * @return Boolean
     */
    public void download(String fileName, String bucketName, HttpServletResponse res) {
        GetObjectArgs objectArgs = GetObjectArgs.builder().bucket(bucketName)
                .object(fileName).build();
        try (GetObjectResponse response = minioClient.getObject(objectArgs)) {
            byte[] buf = new byte[1024];
            int len;
            try (FastByteArrayOutputStream os = new FastByteArrayOutputStream()) {
                while ((len = response.read(buf)) != -1) {
                    os.write(buf, 0, len);
                }
                os.flush();
                byte[] bytes = os.toByteArray();
                res.setCharacterEncoding("utf-8");
                //设置强制下载不打开
                //res.setContentType("application/force-download");
                res.addHeader("Content-Disposition", "attachment;fileName=" + fileName);
                try (ServletOutputStream stream = res.getOutputStream()) {
                    stream.write(bytes);
                    stream.flush();
                }
            }
        } catch (Exception e) {
            log.error("Minio操作时出错：文件下载时：" + e.getMessage() );
            e.printStackTrace();
        }
    }

    /**
     * 文件下载
     *
     * @param fileName 文件名称
     * @param filePath 下载路径
     * @return Boolean
     */
    public Boolean download(String fileName, String bucketName, String filePath) throws IOException, InvalidKeyException, InvalidResponseException, InsufficientDataException, NoSuchAlgorithmException, ServerException, InternalException, XmlParserException, ErrorResponseException {
        try {
            minioClient.downloadObject(
                    DownloadObjectArgs.builder()
                            .bucket(bucketName)
                            .object(fileName)
                            .filename(filePath)
                            .build());
        } catch (Exception e) {
            e.printStackTrace();
            log.error("Minio操作时出错：文件下载时：" + e.getMessage() );
            return false;
        }
        return true;
    }


    /**
     * 查看文件对象
     *
     * @return 存储bucket内文件对象信息
     */
    public List<Item> listObjects(String bucketName) {
        Iterable<Result<Item>> results = minioClient.listObjects(
                ListObjectsArgs.builder().bucket(bucketName)
                        .build());
        List<Item> items = new ArrayList<>();

        try {
            for (Result<Item> result : results) {
                items.add(result.get());
            }
        } catch (Exception e) {
            log.error("Minio操作时出错：查看文件对象时：" + e.getMessage() );
            e.printStackTrace();
            return null;
        }
        return items;
    }

    /**
     * 删除
     *
     * @param fileName
     * @return
     * @throws Exception
     */
    public boolean remove(String fileName, String bucketName) {
        try {
            minioClient.removeObject(RemoveObjectArgs.builder().bucket(bucketName).object(fileName).build());
        } catch (Exception e) {
            log.error("Minio操作时出错：删除"+fileName+"时：" + e.getMessage() );
            return false;
        }
        return true;
    }

    /**
     * 批量删除文件对象
     *
     * @param objects 对象名称集合
     * @return List<DeleteError> 错误信息列表
     */
    public List<DeleteError> removeObjects(List<String> objects, String bucketName) {
        List<DeleteError> deleteErrors = new ArrayList<>();
        List<DeleteObject> deleteObjects = objects.stream().map(DeleteObject::new).collect(Collectors.toList());
        Iterable<Result<DeleteError>> results = minioClient.removeObjects(
                RemoveObjectsArgs.builder()
                        .bucket(bucketName)
                        .objects(deleteObjects)
                        .build());
        try {
            for (Result<DeleteError> result : results) {
                DeleteError error = result.get();
                deleteErrors.add(error);
                log.error("删除文件对象[ " + error.objectName() + "]出错！错误信息：" + error.message());
            }
        } catch (Exception e) {
            log.error("Minio操作时出错：批量删除文件对象时：" + e.getMessage() );
            e.printStackTrace();
        }
        return deleteErrors;
    }

    /**
     * 获取文件对象流（没测试）
     */
    public void getObject(String fileName, String bucketName) {

        try {
            GetObjectResponse object = minioClient.getObject(GetObjectArgs.builder()
                    .bucket(bucketName)
                    .object(fileName)
                    .build());
        } catch (ErrorResponseException | InsufficientDataException | InvalidKeyException |
                 InvalidResponseException | IOException | NoSuchAlgorithmException | XmlParserException |
                 InternalException | io.minio.errors.ServerException e) {
            log.error("Minio操作时出错：获取文件对象流时：" + e.getMessage() );
            e.printStackTrace();
        }
    }


    /**
     * 唯一性处理
     *
     * @param fileName
     * @param onlyFlag
     */
    public String onlyParse(String fileName, Boolean onlyFlag) {
        if (onlyFlag == null) {
            onlyFlag = true;
        }
        if (onlyFlag) {
            //生成的文件名为  原文件名_唯一串.后缀名
            fileName = org.springframework.util.StringUtils.getFilename(org.springframework.util.StringUtils.stripFilenameExtension(fileName)) + "_" + UUIDUtils.getUUID() + "." + org.springframework.util.StringUtils.getFilenameExtension(fileName);
            return fileName;
        } else {
            return fileName;
        }
    }
    public String getMinoDatePathFileName(String dateStr, String fileName, Boolean onlyFlag) {
        String fileNameNew = onlyParse(fileName, onlyFlag);
        String sDatePath = "";
        if (!StringUtils.isEmpty(dateStr)) {
            sDatePath = TimeUtil.dataFormat(dateStr, "yyyy/MM/dd")+ "/";
            if (StringUtils.isEmpty(sDatePath)) {
                sDatePath = TimeUtil.getNowTime("yyyy/MM/dd")+ "/";
            }
        } else {
            sDatePath = TimeUtil.getNowTime("yyyy/MM/dd")+ "/";
        }
        return sDatePath + fileNameNew;
    }
    /**
     * 获取图片可访问url
     * @param objectName
     * @param bucketName
     * @return
     * @throws IOException
     * @throws InvalidKeyException
     * @throws InvalidResponseException
     * @throws InsufficientDataException
     * @throws NoSuchAlgorithmException
     * @throws ServerException
     * @throws InternalException
     * @throws XmlParserException
     * @throws ErrorResponseException
     */
    public String getPresignedObjectUrl(String objectName, String bucketName,Integer expireDayCount) throws IOException, InvalidKeyException, InvalidResponseException, InsufficientDataException, NoSuchAlgorithmException, ServerException, InternalException, XmlParserException, ErrorResponseException, io.minio.errors.ServerException, InternalException {
        String url = "";
        if (expireDayCount == null) {
            expireDayCount = 60 * 60 * 21 * 7;
        }
        url = minioClient.getPresignedObjectUrl(
                GetPresignedObjectUrlArgs.builder()
                        .method(Method.GET)
                        .bucket(bucketName)
                        .object(objectName)
                        .expiry(expireDayCount)
                        .build());
        return url;
    }


    /**
     * 判断文件是否存在(待测试)
     *
     * @param filePath   文件路径
     * @param bucketName 桶名
     * @return true:存在  false:不存在
     */
    public Boolean fileExists(String filePath, String bucketName) {
        try {
            StatObjectResponse statObjectResponse = minioClient.statObject(StatObjectArgs.builder().bucket(bucketName).object(filePath).build());
            if (statObjectResponse != null) {
                return true;
            }
        } catch (Exception e) {
            log.error("Minio操作时出错：判断文件是否存在时：" + e.getMessage() );
        }
        return false;
    }
}