package com.cuit.solarenergy.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cuit.solarenergy.entity.TSiteBaseInfo;
import com.cuit.solarenergy.entity.TStreamBaseInfo;
import com.cuit.solarenergy.entity.dto.TimeAndGeoId;
import com.cuit.solarenergy.entity.dto.TimeAndGeoIdAndIndex;
import com.cuit.solarenergy.entity.dto.TimeAndValue;
import com.cuit.solarenergy.entity.vo.req.ChartBaseInfoVoReq;
import com.cuit.solarenergy.entity.vo.req.SiteListGetVoReq;
import com.cuit.solarenergy.entity.vo.ret.*;
import com.cuit.solarenergy.mapper.*;
import com.cuit.solarenergy.service.GeoServerPubService;
import com.cuit.solarenergy.service.TSiteBaseInfoService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author PYJ
 * @description 针对表【t_site_base_info(站点基本信息表)】的数据库操作Service实现
 * @createDate 2023-06-08 11:37:12
 */
@Service
@Slf4j
public class TSiteBaseInfoServiceImpl implements TSiteBaseInfoService {
    @Resource
    private TSiteBaseInfoMapper tSiteBaseInfoMapper;

    @Resource
    private TSamplingIndexMapper tSamplingIndexMapper;

    @Resource
    private GeoServerPubService geoServerPubService;

    @Resource
    private TStreamBaseInfoMapper tStreamBaseInfoMapper;

    @Resource
    private TSamplingDataDetailsMapper tSamplingDataDetailsMapper;

    @Resource
    private UavInvDataMapper uavInvDataMapper;

    @Resource
    private SatelliteInvDataMapper satelliteInvDataMapper;

    /**
     * 赤道半径
     */
    private static double EARTH_RADIUS = 6378.137;

    private static double rad(double d) {
        return d * Math.PI / 180.0;
    }

    /**
     * @param lon
     * @param lat
     * @Description: 统计分析模块根据经纬度查询最近的三个站点
     * @return: List<TSiteBaseInfo>
     * @Author: LuoSir
     * @Date: 2023/7/17
     */
    @Override
    public List<DataBaseSeparation> getList(Double lon, Double lat) {
        //封装所有距离的集合
        List<DataBaseSeparation> dataBaseSeparationList = new ArrayList<>();
        //1.先查询到所有站点的经纬度封装成一个tSiteBaseInfoList
        List<TSiteBaseInfo> tSiteBaseInfoList = new ArrayList<>();
        tSiteBaseInfoList = tSiteBaseInfoMapper.getSiteBaseInfoBySeparation();
        //2.循环tSiteBaseInfoList，与传入的经纬度进行计算距离，封装到DistanceList
        for (TSiteBaseInfo tSiteBaseInfo : tSiteBaseInfoList) {
            //传入的经度
            double radlon1 = rad(lon);
            //目的地经度
            double radlon2 = rad(tSiteBaseInfo.getLon());
            //传入的纬度
            double radLat1 = rad(lat);
            //目的地纬度
            double radLat2 = rad(tSiteBaseInfo.getLat());
            double a = radLat1 - radLat2;
            double b = radlon1 - radlon2;
            //进行计算
            double s = 2 * Math.asin(Math.sqrt(Math.pow(Math.sin(a / 2), 2)
                    + Math.cos(radLat1) * Math.cos(radLat2)
                    * Math.pow(Math.sin(b / 2), 2)));
            s = s * EARTH_RADIUS;
            // 保留两位小数
            s = Math.round(s * 100d) / 100d;
            s = (s * 1000);

            DataBaseSeparation dataBaseSeparation = DataBaseSeparation.builder()
                    .stationNo(tSiteBaseInfo.getStationNo())
                    .distance(s)
                    .build();
//            dataBaseSeparation.setStationNo(tSiteBaseInfo.getStationNo());
//            dataBaseSeparation.setDistance(s);
            dataBaseSeparationList.add(dataBaseSeparation);
        }
        //3.DistanceList进行排序，取前三个距离最近的数据
        dataBaseSeparationList.sort(Comparator.comparing(DataBaseSeparation::getDistance));
        return dataBaseSeparationList.subList(0, 3);
    }
//
//    /**
//     * @param statisticQueryVoReq
//     * @Description: 根据站点、水质参数获取曲线所需的列表
//     * @return:
//     * @Author: LuoSir
//     * @Date: 2023/8/7
//     */
//    @Override
//    public List<StatisticQueryVoRet> queryDataCurveByParams(StatisticQueryVoReq statisticQueryVoReq) {
//        List<StatisticQueryVoRet> statisticQueryVoRetList;
//        //判断采样点和水质参数不为空
//        if (statisticQueryVoReq.getSiteIdList().isEmpty() || statisticQueryVoReq.getParamsList().isEmpty()) {
//            throw new ParamsException("请选择正确的采样点或者水质参数...");
//        }
//        statisticQueryVoRetList = tSiteBaseInfoMapper.queryDataCurveByParams(statisticQueryVoReq.getParamsList().stream().collect(Collectors.joining(","))
//                , statisticQueryVoReq.getSiteIdList().stream().collect(Collectors.joining(",")));
//        return statisticQueryVoRetList;
//    }

    /**
     * 返回所有站点经纬度
     */
    @Override
    public List<SiteBaseInfoVoRet> getAllSiteList() throws Exception {
        log.info("开始获取所有站点经纬度信息！");
        List<SiteBaseInfoVoRet> list = tSiteBaseInfoMapper.getAllSiteList();
        if (list != null) {
            log.info("获取到所有站点经纬度信息：{}", list);
            return list;
        }
        throw new Exception("获取所有站点经纬度信息列表为空！");
    }

    /**
     * 返回绘制图表的所有信息
     */
    public List<StatisticAnalysisVoRet> getAllChartList(ChartBaseInfoVoReq chartBaseInfoVoReq) throws Exception {
        Page<ChartBaseInfoVoRet> page = new Page<>(chartBaseInfoVoReq.getCurrent(), chartBaseInfoVoReq.getSize());
        List<ChartBaseInfoVoRet> chartBaseInfoVoRetList = new LinkedList<>();
        List<ChartBaseInfoVoRet> sitesList;
        List<StatisticAnalysisVoRet> statisticAnalysisVoRets = new LinkedList<>();

        //1. 获取流域
        String[] site = chartBaseInfoVoReq.getSiteList().get(0).split("-");
        if (site.equals(null)) {
            throw new Exception("获取流域编号失败！");
        }
        String streamCode = site[0];
        LambdaQueryWrapper<TStreamBaseInfo> streamWrapper = new LambdaQueryWrapper<>();
        streamWrapper.eq(TStreamBaseInfo::getStreamCode, streamCode);
        TStreamBaseInfo stream = tStreamBaseInfoMapper.selectOne(streamWrapper);
        if (stream.equals(null)) {
            throw new Exception("获取流域失败！");
        }
        //流域名称
        String streamName = stream.getStreamName();

        //2. 获取采样点图表数据
        if (CollectionUtils.isNotEmpty(chartBaseInfoVoReq.getIndexList())) {
            sitesList = tSamplingIndexMapper.getSitesChartList(chartBaseInfoVoReq.getSiteList(), chartBaseInfoVoReq.getIndexList(), chartBaseInfoVoReq.getStartTime(), chartBaseInfoVoReq.getEndTime());
            if (CollectionUtils.isNotEmpty(sitesList)) {
                chartBaseInfoVoRetList.addAll(sitesList);
            }
        }
        //3. 获取无人机原始影像图表数据
        if (!StringUtils.isEmpty(chartBaseInfoVoReq.getStatisticAnalysisUAVVoReq().getUavIR())) {
            String IR = chartBaseInfoVoReq.getStatisticAnalysisUAVVoReq().getUavIR();
            if (IR.startsWith("UAV-")) {
                IR = IR.substring(4);
            }
            List<TimeAndGeoId> uavIRList = tSamplingIndexMapper.getUAVOriChartList(streamCode, IR, chartBaseInfoVoReq.getStartTime(), chartBaseInfoVoReq.getEndTime());
            if (CollectionUtils.isNotEmpty(uavIRList)) {

                //存放ids
                List<Long> list = new LinkedList<>();
                for (TimeAndGeoId timeAndGeoId : uavIRList) {
                    list.add(timeAndGeoId.getGeoserverId());
                }
                Map<Long, Double> rasterValue = geoServerPubService.getRasterValue(list, chartBaseInfoVoReq.getLon(), chartBaseInfoVoReq.getLat());
                for (TimeAndGeoId timeAndGeoId : uavIRList) {
                    ChartBaseInfoVoRet tempForOne = ChartBaseInfoVoRet.builder()
                            .chartName(streamName + "-无人机-红外波段")
                            .chartValue(rasterValue.get(timeAndGeoId.getGeoserverId()))
                            .chartTime(timeAndGeoId.getCreateTime())
                            .build();

                    if (!Double.isNaN(rasterValue.get(timeAndGeoId.getGeoserverId()))) {
                        if (Math.abs(rasterValue.get(timeAndGeoId.getGeoserverId()).doubleValue()) < Math.pow(Math.PI, 38)) {
                            chartBaseInfoVoRetList.add(tempForOne);
                        }
                    }

                }
            }
        }
        if (!StringUtils.isEmpty(chartBaseInfoVoReq.getStatisticAnalysisUAVVoReq().getUavNIR())) {
            String NIR = chartBaseInfoVoReq.getStatisticAnalysisUAVVoReq().getUavNIR();
            if (NIR.startsWith("UAV-")) {
                NIR = NIR.substring(4);
            }
            List<TimeAndGeoId> uavNIRList = tSamplingIndexMapper.getUAVOriChartList(streamCode,NIR, chartBaseInfoVoReq.getStartTime(), chartBaseInfoVoReq.getEndTime());
            if (CollectionUtils.isNotEmpty(uavNIRList)) {
                //存放ids
                List<Long> list = new LinkedList<>();
                for (TimeAndGeoId timeAndGeoId : uavNIRList) {
                    list.add(timeAndGeoId.getGeoserverId());
                }
                Map<Long, Double> rasterValue = geoServerPubService.getRasterValue(list, chartBaseInfoVoReq.getLon(), chartBaseInfoVoReq.getLat());
                for (TimeAndGeoId timeAndGeoId : uavNIRList) {
                    ChartBaseInfoVoRet tempForOne = ChartBaseInfoVoRet.builder()
                            .chartName(streamName + "-无人机-近红外波段")
                            .chartValue(rasterValue.get(timeAndGeoId.getGeoserverId()))
                            .chartTime(timeAndGeoId.getCreateTime())
                            .build();
                    if (!Double.isNaN(rasterValue.get(timeAndGeoId.getGeoserverId()))) {
                        if (Math.abs(rasterValue.get(timeAndGeoId.getGeoserverId()).doubleValue()) < Math.pow(Math.PI, 38)) {
                            chartBaseInfoVoRetList.add(tempForOne);
                        }
                    }
                }
            }
        }
        //4. 获取无人机反演数据图表数据
        if (CollectionUtils.isNotEmpty(chartBaseInfoVoReq.getStatisticAnalysisUAVVoReq().getUavIndexList())) {
            List<TimeAndGeoIdAndIndex> uavInvList = tSamplingIndexMapper.getUAVInvChartList(streamCode, chartBaseInfoVoReq.getStatisticAnalysisUAVVoReq().getUavIndexList(), chartBaseInfoVoReq.getStartTime(), chartBaseInfoVoReq.getEndTime());
            if (CollectionUtils.isNotEmpty(uavInvList)) {
                //存放ids
                List<Long> list = new LinkedList<>();
                for (TimeAndGeoIdAndIndex timeAndGeoIdAndIndex : uavInvList) {
                    list.add(timeAndGeoIdAndIndex.getGeoserverId());
                }
                Map<Long, Double> rasterValue = geoServerPubService.getRasterValue(list, chartBaseInfoVoReq.getLon(), chartBaseInfoVoReq.getLat());
                for (TimeAndGeoIdAndIndex timeAndGeoIdAndIndex : uavInvList) {
                    ChartBaseInfoVoRet tempForOne = ChartBaseInfoVoRet.builder()
                            .chartName(streamName + "-无人机-" + timeAndGeoIdAndIndex.getIndexName())
                            .chartValue(rasterValue.get(timeAndGeoIdAndIndex.getGeoserverId()))
                            .chartTime(timeAndGeoIdAndIndex.getCreateTime())
                            .build();
                    if (!Double.isNaN(rasterValue.get(timeAndGeoIdAndIndex.getGeoserverId()))) {
                        if (Math.abs(rasterValue.get(timeAndGeoIdAndIndex.getGeoserverId()).doubleValue()) < Math.pow(Math.PI, 38)) {
                            chartBaseInfoVoRetList.add(tempForOne);
                        }
                    }
                }
            }
        }
        //5. 获取卫星遥感原始影像图表数据
        if (!StringUtils.isEmpty(chartBaseInfoVoReq.getStatisticAnalysisSATVoReq().getSatIR())) {
            String IR = chartBaseInfoVoReq.getStatisticAnalysisSATVoReq().getSatIR();
            if (IR.startsWith("SAT-")) {
                IR = IR.substring(4);
            }
            List<TimeAndGeoId> satIRList = tSamplingIndexMapper.getSATOriChartList(chartBaseInfoVoReq.getStatisticAnalysisSATVoReq().getSatellite(), streamCode, IR, chartBaseInfoVoReq.getStartTime(), chartBaseInfoVoReq.getEndTime());
            if (CollectionUtils.isNotEmpty(satIRList)) {

                //存放ids
                List<Long> list = new LinkedList<>();
                for (TimeAndGeoId timeAndGeoId : satIRList) {
                    list.add(timeAndGeoId.getGeoserverId());
                }
                Map<Long, Double> rasterValue = geoServerPubService.getRasterValue(list, chartBaseInfoVoReq.getLon(), chartBaseInfoVoReq.getLat());
                for (TimeAndGeoId timeAndGeoId : satIRList) {
                    ChartBaseInfoVoRet tempForOne = ChartBaseInfoVoRet.builder()
                            .chartName(streamName + "-卫星遥感-红外波段")
                            .chartValue(rasterValue.get(timeAndGeoId.getGeoserverId()))
                            .chartTime(timeAndGeoId.getCreateTime())
                            .build();
                    if (!Double.isNaN(rasterValue.get(timeAndGeoId.getGeoserverId()))) {
                        if (Math.abs(rasterValue.get(timeAndGeoId.getGeoserverId()).doubleValue()) < Math.pow(Math.PI, 38)) {
                            chartBaseInfoVoRetList.add(tempForOne);
                        }
                    }
                }

            }
        }
        if (!StringUtils.isEmpty(chartBaseInfoVoReq.getStatisticAnalysisSATVoReq().getSatNIR())) {
            String NIR = chartBaseInfoVoReq.getStatisticAnalysisSATVoReq().getSatNIR();
            if (NIR.startsWith("SAT-")) {
                NIR = NIR.substring(4);
            }
            List<TimeAndGeoId> satNIRList = tSamplingIndexMapper.getSATOriChartList(chartBaseInfoVoReq.getStatisticAnalysisSATVoReq().getSatellite(), streamCode, NIR, chartBaseInfoVoReq.getStartTime(), chartBaseInfoVoReq.getEndTime());
            if (CollectionUtils.isNotEmpty(satNIRList)) {

                //存放ids
                List<Long> list = new LinkedList<>();
                for (TimeAndGeoId timeAndGeoId : satNIRList) {
                    list.add(timeAndGeoId.getGeoserverId());
                }
                Map<Long, Double> rasterValue = geoServerPubService.getRasterValue(list, chartBaseInfoVoReq.getLon(), chartBaseInfoVoReq.getLat());
                for (TimeAndGeoId timeAndGeoId : satNIRList) {
                    ChartBaseInfoVoRet tempForOne = ChartBaseInfoVoRet.builder()
                            .chartName(streamName + "-卫星遥感-近红外波段")
                            .chartValue(rasterValue.get(timeAndGeoId.getGeoserverId()))
                            .chartTime(timeAndGeoId.getCreateTime())
                            .build();
                    if (!Double.isNaN(rasterValue.get(timeAndGeoId.getGeoserverId()))) {
                        if (Math.abs(rasterValue.get(timeAndGeoId.getGeoserverId()).doubleValue()) < Math.pow(Math.PI, 38)) {
                            chartBaseInfoVoRetList.add(tempForOne);
                        }
                    }
                }

            }
        }
        if (CollectionUtils.isNotEmpty(chartBaseInfoVoReq.getStatisticAnalysisSATVoReq().getSatIndexList())) {
            List<TimeAndGeoIdAndIndex> satInvList = tSamplingIndexMapper.getSATInvChartList(chartBaseInfoVoReq.getStatisticAnalysisSATVoReq().getSatellite(), streamCode, chartBaseInfoVoReq.getStatisticAnalysisUAVVoReq().getUavIndexList(), chartBaseInfoVoReq.getStartTime(), chartBaseInfoVoReq.getEndTime());
            if (CollectionUtils.isNotEmpty(satInvList)) {
                //存放ids
                List<Long> list = new LinkedList<>();
                for (TimeAndGeoIdAndIndex timeAndGeoIdAndIndex : satInvList) {
                    list.add(timeAndGeoIdAndIndex.getGeoserverId());
                }
                Map<Long, Double> rasterValue = geoServerPubService.getRasterValue(list, chartBaseInfoVoReq.getLon(), chartBaseInfoVoReq.getLat());
                for (TimeAndGeoIdAndIndex timeAndGeoIdAndIndex : satInvList) {
                    ChartBaseInfoVoRet tempForOne = ChartBaseInfoVoRet.builder()
                            .chartName(streamName + "-卫星遥感-" + timeAndGeoIdAndIndex.getIndexName())
                            .chartValue(rasterValue.get(timeAndGeoIdAndIndex.getGeoserverId()))
                            .chartTime(timeAndGeoIdAndIndex.getCreateTime())
                            .build();
                    if (!Double.isNaN(rasterValue.get(timeAndGeoIdAndIndex.getGeoserverId()))) {
                        if (Math.abs(rasterValue.get(timeAndGeoIdAndIndex.getGeoserverId()).doubleValue()) < Math.pow(Math.PI, 38)) {
                            chartBaseInfoVoRetList.add(tempForOne);
                        }
                    }
                }
            }
        }
        //将所有数据以名称分类（方便前端展示）
        if (CollectionUtils.isNotEmpty(chartBaseInfoVoRetList)) {
            //取出第一条数据的名称
            String chartName = chartBaseInfoVoRetList.get(0).getChartName();
            //存放同一名称下的所有时间和数据值
            LinkedList<TimeAndValue> timeAndValueLinkedList = new LinkedList<>();
            for (int i = 0; i < chartBaseInfoVoRetList.size(); i++) {
                //同一名称的数据
                if (chartBaseInfoVoRetList.get(i).getChartName().equals(chartName)) {
                    TimeAndValue timeAndValue = TimeAndValue.builder()
                            .time(chartBaseInfoVoRetList.get(i).getChartTime())
                            .value(chartBaseInfoVoRetList.get(i).getChartValue())
                            .build();
                    timeAndValueLinkedList.add(timeAndValue);
                    if (i == chartBaseInfoVoRetList.size() - 1) {
                        statisticAnalysisVoRets.add(StatisticAnalysisVoRet.builder().name(chartName).timeAndValueList(timeAndValueLinkedList).build());
                    }
                } else {

                    statisticAnalysisVoRets.add(StatisticAnalysisVoRet.builder().name(chartName).timeAndValueList(timeAndValueLinkedList).build());
                    chartName = chartBaseInfoVoRetList.get(i).getChartName();
                    //清空timeAndValueLinkedList
                    timeAndValueLinkedList = new LinkedList<>();
                    TimeAndValue timeAndValue = TimeAndValue.builder().time(chartBaseInfoVoRetList.get(i).getChartTime()).value(chartBaseInfoVoRetList.get(i).getChartValue()).build();
                    timeAndValueLinkedList.add(timeAndValue);
                    if (i == chartBaseInfoVoRetList.size() - 1) {
                        statisticAnalysisVoRets.add(StatisticAnalysisVoRet.builder().name(chartName).timeAndValueList(timeAndValueLinkedList).build());
                    }
                }
            }
        }
        return statisticAnalysisVoRets;
    }

    @Override
    public PageVo<TSiteBaseInfo> getSiteList(SiteListGetVoReq req) {
        Page<TSiteBaseInfo> page = new Page<>(req.getCurrent(), req.getSize());
        String streamCode = null;
        if (StringUtils.isNotEmpty(req.getStreamName())) {
            // 根据流域名称查询对应的代码
            LambdaQueryWrapper<TStreamBaseInfo> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(TStreamBaseInfo::getStreamName, req.getStreamName());
            TStreamBaseInfo tStreamBaseInfo = tStreamBaseInfoMapper.selectOne(queryWrapper);
            streamCode = tStreamBaseInfo.getStreamCode();
        }
        page = tSiteBaseInfoMapper.getList(page, streamCode, req.getRelativeLocation(), req.getKeywords());
        return new PageVo<>(page);
    }

    /**
     * 返回指定流域指定水质参数各站点数值信息
     * @param streamName 流域名称
     * @param waterParameter 水质参数
     * @return
     */
    @Override
    public List<SiteAndValueVoRet> getValueOfSite(String streamName, String waterParameter) {
        return tSiteBaseInfoMapper.getValueOfSite(streamName, waterParameter);
    }

    /**
     * 返回指定流域近三年的站点统计信息
     * @param streamName 流域名称
     * @return
     */
    @Override
    public List<YearAndCountVoRet> getAllInfoForThreeYear(String streamName) {
        return tSamplingDataDetailsMapper.getAllInfoForThreeYear(streamName);
    }

    /**
     * 返回无人机各年月份统计信息
     * @param streamName  流域名称
     * @return
     */
    @Override
    public List<MonthAndCountVoRet> getCountOfUAV(String streamName) {
        return uavInvDataMapper.getCountOfUAV(streamName);
    }

    /**
     * 返回卫星遥感各年月份统计信息
     * @param streamName 流域名称
     * @return
     */
    @Override
    public List<MonthAndCountVoRet> getCountOfSAT(String streamName) {
        return satelliteInvDataMapper.getCountOfSAT(streamName);
    }
}





